/**
 * 网络访问模块，前后端代码共享
 */

const axios = require('axios')

const request = axios.create({
    timeout: 10000
})

async function get(url,opt,times) {
    // 响应模板
    const ret = {
        // 完全符合预期
        OK: false,
        // 请求失败，一般是网络问题
        FAIL: false,
        // 服务器响应错误，状态码不是 2xx
        ERROR: false,
        // 因为配置出错导致无法请求
        INVALID: false,
        // 服务器正常响应，但是返回数据非业务逻辑预期之内
        UNEXPECTED: false,
        // 数据
        data: null,
        // 信息（一般是错误提示）
        message: null,
        // 错误信息
        error: null
    }
    // 重试次数
    if (times === undefined) times = 1
    const serverName = (opt && opt.serverName && opt.serverName !== '') ? ` [${opt.serverName}] ` : ''
    try {
        const res = await request.get(url,opt)
        // 外层响应码都是 2xx
        // 检查 data 内容
        // 内层代码大于 200，返回提示信息
        if (res.data.code > 200) {
            ret.UNEXPECTED = true
            ret.message = `服务器${serverName}返回结果：${res.data.status}，提示信息：${res.data.message}`
            return ret
        }
        // 一切正常，直接返回内层 data
        ret.OK = true
        ret.data = res.data.data
        return ret
    }
    catch (error) {
        // 区分情况进行处理
        if (error.response) {
            // 请求已经发出，服务器也给出了响应，但是状态码不是 2XX
            // console.log(error.response)
            ret.error = error.response
            // 错误码大于 500 时启动重试机制
            if (error.request.status > 500) {
                if (times < 3) {
                    times = times + 1
                    return await get(url,opt,times)
                }
            }
            ret.ERROR = true
            ret.message = `服务器${serverName}返回错误代码：${error.response.status}，提示信息：${error.response.data.message}`
            return ret
        }
        else if (error.request) {
            // 请求已经发出，但是没有收到 response
            // console.log(error.request)
            ret.error = error.request
            // 错误码大于 500 时启动重试机制
            if (times < 3) {
                times = times + 1
                return await get(url,opt,times)
            }
            ret.FAIL = true
            ret.message = `服务器${serverName}无响应或拒绝服务`
            return ret
        }
        else {
            // 请求配置出错
            // console.log('Error', error.message)
            ret.error = error.message
            ret.INVALID = true
            ret.message = `针对服务器${serverName}的网络请求参数有误，请检查`
            return ret
        }
    }
}

async function post(url,payload,opt,times) {
    // 响应模板
    const ret = {
        // 完全符合预期
        OK: false,
        // 请求失败，一般是网络问题
        FAIL: false,
        // 服务器响应错误，状态码不是 2xx
        ERROR: false,
        // 因为配置出错导致无法请求
        INVALID: false,
        // 服务器正常响应，但是返回数据非业务逻辑预期之内
        UNEXPECTED: false,
        // 数据
        data: null,
        // 信息（一般是错误提示）
        message: null,
        // 错误信息
        error: null
    }
    // 重试次数
    if (times === undefined) times = 1
    const serverName = (opt && opt.serverName && opt.serverName !== '') ? ` [${opt.serverName}] ` : ''
    try {
        const res = await request.post(url,payload,opt)
        // 外层响应码都是 2xx
        // 检查 data 内容
        // 内层代码大于 200，返回提示信息
        if (res.data.code > 200) {
            ret.UNEXPECTED = true
            ret.message = `服务器${serverName}返回结果：${res.data.status}，提示信息：${res.data.message}`
            return ret
        }
        // 一切正常，直接返回内层 data
        ret.OK = true
        ret.data = res.data.data
        return ret
    }
    catch (error) {
        // 区分情况进行处理
        if (error.response) {
            // 请求已经发出，服务器也给出了响应，但是状态码不是 2XX
            // console.log(error.response)
            ret.error = error.response
            // 错误码大于 500 时启动重试机制
            if (error.request.status > 500) {
                if (times < 3) {
                    times = times + 1
                    return await post(url,payload,opt,times)
                }
            }
            ret.ERROR = true
            ret.message = `服务器${serverName}返回错误代码：${error.response.status}，提示信息：${error.response.data.message}`
            return ret
        }
        else if (error.request) {
            // 请求已经发出，但是没有收到 response
            // console.log(error.request)
            ret.error = error.request
            // 错误码大于 500 时启动重试机制
            if (times < 3) {
                times = times + 1
                return await post(url,payload,opt,times)
            }
            ret.FAIL = true
            ret.message = `服务器${serverName}无响应或拒绝服务`
            return ret
        }
        else {
            // 请求配置出错
            // console.log('Error', error.message)
            ret.error = error.message
            ret.INVALID = true
            ret.message = `针对服务器${serverName}的网络请求参数有误，请检查`
            return ret
        }
    }
}

module.exports = {
    get,
    post
}